@@ -8,6 +8,7 @@
 #include <vector>
 #include <stdlib.h>
 #include "crossprodmat.h"
+#include "covariancemat.h"
 #include "Polynomial.h"
 
 
@@ -40,17 +41,6 @@ extern "C" {
 
   SEXP pimomMarginalKI(SEXP Ssel, SEXP Snsel, SEXP Sn, SEXP Sp, SEXP Sy, SEXP Ssumy2, SEXP SXtX, SEXP SytX, SEXP Sphi, SEXP Stau, SEXP Smethod, SEXP SB, SEXP Slogscale, SEXP Sngroups, SEXP Snvaringroup);
   SEXP pimomMarginalUI(SEXP Ssel, SEXP Snsel, SEXP Sn, SEXP Sp, SEXP Sy, SEXP Ssumy2, SEXP Sx, SEXP SXtX, SEXP SytX, SEXP Stau, SEXP Smethod, SEXP SB, SEXP Slogscale, SEXP Salpha, SEXP Slambda, SEXP Sngroups, SEXP Snvaringroup);
-
-  SEXP pemomMarginalUI(SEXP Ssel, SEXP Snsel, SEXP Sn, SEXP Sp, SEXP Sy, SEXP Ssumy2, SEXP Sx, SEXP SXtX, SEXP SytX, SEXP Stau, SEXP Smethod, SEXP SB, SEXP Slogscale, SEXP Salpha, SEXP Slambda, SEXP Sngroups, SEXP Snvaringroup);
-
-  //Integrated likelihood for linear models under Zellner's prior
-  SEXP zellnerMarginalKI(SEXP Ssel, SEXP Snsel, SEXP Sn, SEXP Sp, SEXP Sy, SEXP Ssumy2, SEXP SXtX, SEXP SytX, SEXP Sphi, SEXP Stau, SEXP Slogscale, SEXP Sngroups, SEXP Snvaringroup);
-  SEXP zellnerMarginalUI(SEXP Ssel, SEXP Snsel, SEXP Sn, SEXP Sp, SEXP Sy, SEXP Ssumy2, SEXP Sx, SEXP SXtX, SEXP SytX, SEXP Stau, SEXP Slogscale, SEXP Salpha, SEXP Slambda, SEXP Sngroups, SEXP Snvaringroup);
-
-  //Integrated likelihoods under skew normal, laplace and asymmetric laplace residuals
-  SEXP nlpMarginalSkewNormI(SEXP Ssel, SEXP Snsel, SEXP Sn, SEXP Sp, SEXP Sy, SEXP Ssumy2, SEXP Sx, SEXP SXtX, SEXP SytX, SEXP Stau, SEXP Staualpha, SEXP Sfixatanhalpha, SEXP Sr, SEXP Smethod, SEXP SoptimMethod, SEXP SB, SEXP Slogscale, SEXP Salpha, SEXP Slambda, SEXP SprCoef, SEXP Sngroups, SEXP Snvaringroup);
-  SEXP nlpMarginalAlaplI(SEXP Ssel, SEXP Snsel, SEXP Sn, SEXP Sp, SEXP Sy, SEXP Ssumy2, SEXP Sx, SEXP SXtX, SEXP SytX, SEXP Stau, SEXP Staualpha, SEXP Sfixatanhalpha, SEXP Sr, SEXP Ssymmetric, SEXP Smethod, SEXP Shesstype, SEXP SoptimMethod, SEXP SB, SEXP Slogscale, SEXP Salpha, SEXP Slambda, SEXP SprCoef, SEXP Sngroups, SEXP Snvaringroup);
-
   }
 
 
@@ -73,6 +63,7 @@ struct marginalPars {
   double *x;
   crossprodmat *XtX;  //t(x) %*% x using all observations
   crossprodmat *XtXuncens; //t(x) %*% x using uncensored observations
+  covariancemat *V0inv;  // covariance matrix for coef and groups priors
   double *ytX;             //t(x) %*% y using all observations
   double *ytXuncens;       //t(x) %*% y using uncensored observations
   double *m;  //Sinv * Xty   (needed by mom and emom)
@@ -143,6 +134,7 @@ void testfunction(double *x);
 //Setting prior & marginals
 //*************************************************************************************
 
+int mspriorCode(int *prCoef, int *prGroup, struct marginalPars *pars);
 pt2margFun set_marginalFunction(int *priorcode, int *knownphi, int *family, struct marginalPars *pars);
 pt2margFun set_priorFunction(int *prDelta, int *prConstr, int *family);
 pt2modavgPrior set_priorFunction_modavg(int *priorModel);
@@ -257,6 +249,10 @@ double pemomMarginalUC(int *sel, int *nsel, struct marginalPars *pars);
 double zellnerMarginalKC(int *sel, int *nsel, struct marginalPars *pars);
 double zellnerMarginalUC(int *sel, int *nsel, struct marginalPars *pars);
 
+// Normal on all coef
+double normalidMarginalKC(int *sel, int *nsel, struct marginalPars *pars);
+double normalidMarginalUC(int *sel, int *nsel, struct marginalPars *pars);
+
 // pMOM on individual coef, group MOM on groups
 double pmomgmomMarg(int *sel, int *nsel, struct marginalPars *pars);
 
@@ -272,7 +268,11 @@ double pemomgzellMarg(int *sel, int *nsel, struct marginalPars *pars);
 // Zellner on individual coef, block Zellner on groups
 double zellgzellMarg (int *sel, int *nsel, struct marginalPars *pars);
 
+// Zellner on individual coef, normalid on groups
+double zellnormidMarg (int *sel, int *nsel, struct marginalPars *pars);
 
+// normalid on individual coef, group Zellner on groups
+double normidgzellMarg (int *sel, int *nsel, struct marginalPars *pars);
 
 //*************************************************************************************
 // MARGINAL LIKELIHOOD FOR ACCELERATED FAILURE TIME MODELS
